#include "StdAfx.h"
#include "ChatDialogManager.h"
#include "ChatDlg.h"
#include "NeMessengerDlg.h"
#include "FileTraslateAbnormalHandler.h"
#include "main_frame.hpp"
#include "UIElement.h"

#define CHAT_DIALOG_ALIGN_RATIO 4
#define CHAT_DIALOG_HALF_HEIGHT 2

ChatDialogManager::ChatDialogManager(void)
: m_offset(0)
{
    for (UINT i = 0; i < _countof(m_chatAlignRects); i++)
    {
        m_chatAlignRects[i].SetRectEmpty();
    }
    m_screenRect.SetRectEmpty();
}

ChatDialogManager::~ChatDialogManager(void)
{
}

int ChatDialogManager::GetCountOfChatDlgNotClosed() // when a chat dialog has file-open modal dialog there, it can't be closed.
{
    int closedUnsuccess =0;
	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CChatDlg *pDlg = (CChatDlg*)m_listDialog.GetNext(pos);
		if( pDlg != NULL && pDlg->GetSafeHwnd() )
		{
			if (pDlg->GetOngoingAction() == SENDACTION_FILE)
			{
				pDlg->SetWindowTop();
				pDlg->SetForegroundWindowForce(pDlg->GetSafeHwnd());

				closedUnsuccess ++;
			} 
		}
	}
	if (closedUnsuccess > 0)
	{
		NMessageBox(AfxGetMainWnd(), LoadStringFromResource(IDS_STRINGCHATDLGBUSY), _T(""), MB_OK);

	} 
	return closedUnsuccess;
}


int ChatDialogManager::RemoveAll()
{
	CSingleLock removeLock(&m_csDialogRemove);
	removeLock.Lock();
	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CChatDlg *pDlg = (CChatDlg*)m_listDialog.GetNext(pos);
		if( pDlg != NULL && pDlg->GetSafeHwnd() )
		{		
			pDlg->CloseChatDialog(pDlg->GetNeedRestoreChatRoom(), false);
			if (IsWindow(pDlg->GetHWND()))
			{
				::DestroyWindow(pDlg->GetHWND());
			}
		}
	}
	
    m_listDialog.RemoveAll();
	m_mapRecentChatRoomData.clear();
	removeLock.Unlock();
	return 0;
}

void ChatDialogManager::Remove( CDialog* pDialog, bool bRemoveRecentItem)
{
	if (bRemoveRecentItem)
	{
		GET_MESSENGER_DLG()->GetMainFrame()->RemoveChatRoom(((CChatDlg*)pDialog)->GetRoomID());

		map<int, NE_MYROOM_DATA>::iterator it = m_mapRecentChatRoomData.find(((CChatDlg*)pDialog)->GetRoomID());
		if (it != m_mapRecentChatRoomData.end())
		{
			m_mapRecentChatRoomData.erase(it);
		}
	}
	else
	{
		GET_MESSENGER_DLG()->GetMainFrame()->SetRecentChatDlg(((CChatDlg*)pDialog)->GetRoomID(), NULL);
	}
	
	DialogManager::Remove(pDialog);
}

void ChatDialogManager::SetViewLanguagePack()
{
	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CChatDlg* pDialog =  (CChatDlg*)m_listDialog.GetNext(pos);
		if (pDialog->GetSafeHwnd())
		{
			pDialog->__ResourceLoad();
		}
	}
}

int ChatDialogManager::MakeSequenceID()
{
	TRACE("%s %d\n", __FUNCTION__, __LINE__);

	int nSeqNumber = 0;
	for (; nSeqNumber < MAX_CHAT_USER; nSeqNumber++)
	{
		BOOL bSame = FALSE;
		
		for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
		{
			CChatDlg* pDlg = (CChatDlg*)m_listDialog.GetNext(pos);
			trid_t trid = pDlg->GetChatTrid();
			int oldSeqNumber = trid.GetID() - CHAT_BASE_TRID;
			if (nSeqNumber == oldSeqNumber)
			{
				bSame = TRUE;
				break;
			}
		}

		if (bSame == FALSE)
			break;
	}

	return nSeqNumber;	
}

void ChatDialogManager::ReUserNickname(CChatDlg* pDlg, LPCTSTR strID, LPCTSTR strNickname)
{
	if (pDlg)
	{
		pDlg->ChangeNickNameChatUserList(strID, strNickname);
	}
	else
		ASSERT(FALSE);
}

bool ChatDialogManager::IsAvailableChatRoom()
{
	return true;	//Not limit count of chat window by sevin
	/*
	const int nMaxChatRoomCount = 30;
	bool bReturn = true;

	int nCount = m_listDialog.GetCount();
	if (nCount >= nMaxChatRoomCount)
	{
		//AfxMessageBox(IDS_STRING156);
		bReturn = false;
	}

	return bReturn;
	*/
}

bool ChatDialogManager::IsCreatedSingleChatRoom(const std::wstring& userId)
{
	/* Deprecated. 
	 * Please use FindSingleChatDlg function.
	 */
	bool bReturn = false;

	if (m_listDialog.GetCount() > 0)
	{
		CChatDlg* pDlg = NULL;

		for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
		{
			pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

			if (!pDlg)
				continue;

			if (pDlg->FindUserMap(userId.c_str()) > -1 && 
				pDlg->IsGroupRoom() == FALSE)
			{
				::ShowWindow(pDlg->GetHWND(), SW_SHOWNOACTIVATE);

				// ȭâ Ȱȭ ָ鼭 ȭ Ʈ ߰ش.
				if (GET_MESSENGER_DLG()->GetMainFrame())
					GET_MESSENGER_DLG()->GetMainFrame()->AddChatRoomList(pDlg);

				if (pDlg->IsIconic())
					::ShowWindow(pDlg->GetHWND(), SW_RESTORE);

				bReturn = true;
				pDlg->EnableWindow(TRUE);
				
				pDlg->SetWindowTop();
				pDlg->SetForegroundWindowForce(pDlg->GetSafeHwnd());
			
				break;
			}
		}
	}

	NTALK_LOG->Debug(L"[CHAT] IsCreatedSingleChatRoom %s %s", userId.c_str(), bReturn ? L"SUCC" : L"FAIL");

	return bReturn;
}

CChatDlg* ChatDialogManager::CreateChatDlg( const std::vector<CChatUser*>& userList, EChatMakeType eMakeType , int channel)
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	CChatDlg* pChatDlg = new CChatDlg();

	if (NULL == pChatDlg)
		return NULL;
	pChatDlg->InitChannel(channel); 

	int nSequnceId = MakeSequenceID();
	pChatDlg->SetChatInfo(nSequnceId, userList, eMakeType);

	HWND hWnd = pChatDlg->Create(NULL, _T("ChatDialog"), WS_VISIBLE|WS_OVERLAPPEDWINDOW, WS_EX_STATICEDGE | WS_EX_APPWINDOW, 0, 0, 485, 423);
	::ShowWindow(pChatDlg->GetHWND(), SW_HIDE);

	return pChatDlg;
}

void ChatDialogManager::AddUserListChatDlg(CChatDlg* pChatDlg, std::vector<CChatUser*>& userList)
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	if ((pChatDlg != NULL) && ::IsWindow(pChatDlg->GetSafeHwnd()))
	{
		int nUserCount = 0;
		vector<CChatUser*>::const_iterator itr = userList.begin();
		list<wstring> listUserIds;
		for(; itr != userList.end(); )
		{
			listUserIds.push_back((LPCTSTR)((*itr)->GetID()));
			pChatDlg->AddChatUserToListCtrl(*itr,false);
			if (pChatDlg->GetMyUserID().CompareNoCase((*itr)->GetID()) == 0)
			{
				delete (*itr);
                itr = userList.erase(itr);
				continue;
			}
            else
            {
                ++itr;
            }

			++nUserCount;
		} 


		if (nUserCount == 1)
		{
			pChatDlg->ShowChatList(false);
			pChatDlg->SetRestoreLeftWindow(false);
		}
		else
		{
			pChatDlg->ExpandWindow(true,SW_HIDE);
			pChatDlg->SetRestoreLeftWindow(true);

// 			if (pChatDlg->IsZoomed())
// 			{
// 				pChatDlg->SetInitMaxWindow(true);
// 			}
// 			else
// 			{
// 				pChatDlg->ExpandWindow(true,SW_HIDE);
// 				pChatDlg->SetRestoreLeftWindow(true);
// 			}

			pChatDlg->ShowChatList(true);
		}

		CUserImageActor::Instance()->RequestUserImages(listUserIds);
		m_listDialog.AddTail((CObject*)pChatDlg);
		pChatDlg->JudgeReadOnly();
	}
	else
	{
		delete pChatDlg;
		pChatDlg = NULL;
		ASSERT(FALSE);
	}
}

CChatDlg* ChatDialogManager::FindCreatedSessionChatDlg(int nRoomNo)
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	CChatDlg* pChatDlg = NULL;
	BOOL bFindDlg = FALSE;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pChatDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (!pChatDlg)
			continue;

		if (pChatDlg->GetRoomID() == nRoomNo )
		{
			NTALK_LOG->Debug(L"[CHAT] FindCreatedSessionChatDlg SUCC %d", nRoomNo);
			bFindDlg = TRUE;
			break;
		}
	}

	if (bFindDlg == FALSE)
	{
		NTALK_LOG->Debug(L"[CHAT] FindCreatedSessionChatDlg FAIL %d", nRoomNo);
		pChatDlg = NULL;
	}

	return pChatDlg;
}

/*   1:1 ȭ ã´.
 * nRoomNo -1  ش ڸ  OK. (ȣ  ʴ´)
 */
CChatDlg* ChatDialogManager::FindSingleChatDlg(LPCTSTR userId, int nRoomNo)
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	CChatDlg* pChatDlg = NULL;
	BOOL bFindDlg = FALSE;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pChatDlg = (CChatDlg*)m_listDialog.GetNext(pos);
		
		if (pChatDlg == NULL || pChatDlg->IsGroupRoom())
			continue;

		if (pChatDlg->FindUserMap(userId) > -1)
		{
			if (nRoomNo == -1 || pChatDlg->GetRoomID() == nRoomNo)
			{
                NTALK_LOG->Debug(L"[CHAT] FindSingleChatDlg SUCC %s:%d:%d:0x%x", userId, nRoomNo, 
                    pChatDlg->GetSessionStatus(), pChatDlg->GetHWND());
                
				bFindDlg = TRUE;
				break;
			}
		}
	}

	//Modify by sevin, create from recent chat room list
	if (bFindDlg == FALSE)
	{
		bFindDlg = GET_MESSENGER_DLG()->GetMainFrame()->FindSingleChatDlg(userId, nRoomNo);
		if (bFindDlg)
		{
			map<int, NE_MYROOM_DATA>::iterator it = m_mapRecentChatRoomData.begin();
			for (; it!=m_mapRecentChatRoomData.end(); ++it)
			{
				if ((*it).second.nRoomType == CHATROOM_SINGLETYPE)
				{
					for (size_t i=0; i<(*it).second.listChatMemberData.size(); ++i)
					{
						if ((*it).second.listChatMemberData[i].strID == userId)
						{
							bFindDlg = TRUE;
							pChatDlg = GET_APP()->m_pChatDialogManager->CreateJoinedChat((*it).second);
							break;
						}
					}
				}
			}
		}
		
	}

	if (bFindDlg == FALSE)
	{
		NTALK_LOG->Debug(L"[CHAT] FindSingleChatDlg FAIL %s:%d", userId, nRoomNo);
		pChatDlg = NULL;
	}

	return pChatDlg;
}

CChatDlg* ChatDialogManager::FindCreatedSessionChatDlg(const std::vector<CChatUser *>& userList, LPCTSTR serverAddress, int nPort, int nRoomNo)
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	CChatDlg* pChatDlg = NULL;
	BOOL bFindDlg = FALSE;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pChatDlg = (CChatDlg*)m_listDialog.GetNext(pos);
		
		if (!pChatDlg)
			continue;

		if ( pChatDlg->IsEqualChatDlg(userList, serverAddress, nPort, nRoomNo) == TRUE )
		{
			NTALK_LOG->Debug(L"[CHAT] FindCreatedSessionChatDlg SUCC %s:%d:%d", serverAddress, nPort, nRoomNo);
			bFindDlg = TRUE;
			break;
		}
	}

	if (bFindDlg == FALSE)
	{
		NTALK_LOG->Debug(L"[CHAT] FindCreatedSessionChatDlg FAIL %s:%d:%d", serverAddress, nPort, nRoomNo);
		pChatDlg = NULL;
	}

	return pChatDlg;
}

void ChatDialogManager::CreateBuddyChat(const std::vector<const IGTBuddyUser*>& buddyVector, BUDDY_SENDACTIONType FollowedAction)
{
	if (buddyVector.empty() == true)
		return;

	if (!GET_MESSENGER_DLG()->IsAvailableUpdateStatus())
		return;

	CreateBuddyChatDialog createChatDialog(buddyVector);
	CChatDlg* pdlg = Create(&createChatDialog);
	if (pdlg != NULL)
	{
		BUDDY_SENDACTIONType onGoingAction = pdlg->GetOngoingAction();
		if (SENDACTION_FILE != onGoingAction)
		{
			pdlg->SetOngoingAction(FollowedAction);
			if (SENDACTION_FILE == FollowedAction)
			{
				::PostMessage(pdlg->GetSafeHwnd(),WM_CHATDLGSENDFILE,0,0);
			}
		}
	}
    
}

void ChatDialogManager::CreateMemberChat(const std::vector<NE_MEMBER_DATA>& memberVector)
{	
	CreateMemberChatDialog createChatDialog(memberVector);

	Create(&createChatDialog);
}


void ChatDialogManager::CreateSingleMemberChat( const CString& strUserID, const CString& strUserName )
{	
	CreateSingleMemberChatDialog createChatDialog(strUserID, strUserName);

	Create(&createChatDialog);
}

void ChatDialogManager::CreateSingleBuddyChat( const IGTBuddyUser* pUser )
{	
	CreateSingleBuddyChatDialog createChatDialog(pUser);

	Create(&createChatDialog);
}

void ChatDialogManager::CreateUserIDArrayChat(const CStringArray& userIDArray )
{	
	CreateUserIDArrayChatDialog createChatDialog(userIDArray);

	Create(&createChatDialog);
}

void ChatDialogManager::CreateInvitedChat( CChatUser *pChatUser, const CString& serverAddress, int nPort, int nRoomNo, int nRoomType )
{
    FUNCTION_BLOCK_TRACE(DEFAULT_BLOCK_TRACE_THRESHOLD);

	CSingleLock lock(&m_csFindCreatedDlg);
	lock.Lock();

	CChatDlg* pChatDlg = NULL;

	//   1:1 ȭ ִ  ʴ Ŷ Ѵ.(1:1 嵵 )
	if (nRoomType == CHATROOM_SINGLETYPE)
	{
		CSingleLock removeLock(&m_csDialogRemove);
		removeLock.Lock();	// Find   dialog  ʵ removeLock ´.

		pChatDlg = FindSingleChatDlg(pChatUser->GetID());

		removeLock.Unlock();

		if (pChatDlg)
		{
			lock.Unlock();

			SAFE_DELETE(pChatUser);
			return;
		}
	}

	CSingleLock removeChatLock(&m_csDialogRemove);
	removeChatLock.Lock();

	//   ȭâ ִ Ȯ
	pChatDlg = FindCreatedSessionChatDlg(nRoomNo);
	if (pChatDlg)
	{
		//  츸
		if( TRUE == pChatDlg->GetForceLeave())	// DIM ó ° ƴ϶  ʴ  ϵ Ѵ.(׷ ȭ, 1:1 ȭ   ó)
		{
			SAFE_DELETE(pChatUser);

			pChatDlg->RemoveAllUser();	//  ȭ Ʈ (ο ä  ξ ٽ ´.    Դ   ̹Ƿ)
            pChatDlg->JudgeReadOnly();
            //pChatDlg->SetReadOnly(FALSE, FALSE);
			//pChatDlg->SetUIReadOnly(FALSE);
			pChatDlg->EnableDragAndDrop(TRUE);
			pChatDlg->SetChatRoomMakeType(MAKETYPE_JOIN);

			bool result = pChatDlg->NegotiateChatSession();

			if(result != true)
				pChatDlg->CloseChatDialog();
			else
			{
				pChatDlg->SetSessionRetryAvailable(TRUE);
				pChatDlg->SetForceLeave(FALSE);
				pChatDlg->RefreshTitle();
			}
		}
	}
	else	// ο ȭâ ؾ ϴ 
	{
		std::vector<CChatUser *> userVector;
		userVector.push_back(pChatUser);
		pChatDlg = CreateChatDlg(userVector, MAKETYPE_INVITE, 1);

		if (NULL != pChatDlg)
		{
			NTALK_LOG->Debug(L"[CHAT] MAKETYPE_INVITE, new");
			pChatDlg->SetInviteUserName(pChatUser->GetName());
			pChatDlg->SetServerInfo(serverAddress, nPort, nRoomNo, _T("new"));	
			pChatDlg->SetSessionReason(REASON_CREATE);
			pChatDlg->ResetCreateSessionCheckTimes();
			if (nRoomType == CHATROOM_MULTITYPE)
			{
				pChatDlg->SetGroupRoom(TRUE);
			}
			AddUserListChatDlg(pChatDlg, userVector);
			bool result = pChatDlg->NegotiateChatSession();
			if(result != true)
				pChatDlg->CloseChatDialog();

			/* ʴ ȭ ⼭ ٷ   ȭ   â .
			 * ׷ ȭâ  ȭ Ͽ ߰ϴ  ȭ Ž óѴ.
			 */
		}
	}

	removeChatLock.Unlock();
	lock.Unlock();
}

// ̹   äâ . OnGTWASActor_ResponseGetMyRoomList()  ȣ
CChatDlg* ChatDialogManager::CreateJoinedChat(NE_MYROOM_DATA& myRoomData)
{
    NTALK_LOG->Debug(L"[CHAT] CREATEJOINEDCHAT, %d:%d:%d:%d:%d", 
        myRoomData.nRoomID, myRoomData.nRoomType, myRoomData.nLastMsgSeq, myRoomData.nLastMsgTrid, myRoomData.listChatMemberData.size());

 	std::vector<NE_CHATMEMBER_DATA>& memberVector = myRoomData.listChatMemberData;

	if (memberVector.empty() == true)
		return NULL;

	std::vector<CChatUser *> userVector;
	CString strLastMsgUserID = myRoomData.strLastMsgUserId.c_str();
	CString strLastMsgUserName = _T("");
	CString strLastMsgUserTalkName = _T("");


	std::vector<NE_CHATMEMBER_DATA>::iterator member_iter = memberVector.begin();

	if (member_iter == memberVector.end())
		return NULL;	

	for (; member_iter != memberVector.end(); ++member_iter)
	{
		NE_CHATMEMBER_DATA chatMemberData = *member_iter;					
		CChatUser* pUser = new CChatUser();
		if (NULL != pUser)
		{
			pUser->SetID(chatMemberData.strID.c_str());
			pUser->SetName(chatMemberData.strName.c_str());
			pUser->SetNickName(chatMemberData.strNickName.c_str());

            GTUserStatusType userStatus = ustOFFLINE;
            const IGTBuddyUser* pMsgUser = g_pMessengerActor->GetAvailbeBuddyInfo(chatMemberData.strID.c_str());
			if(pMsgUser)
			{
                userStatus = pMsgUser->GetStatus();
			}
            pUser->SetStatus(userStatus);

			userVector.push_back(pUser);

			if (pUser->GetID().CompareNoCase(strLastMsgUserID) == 0)
			{
				strLastMsgUserName = pUser->GetName();
				strLastMsgUserTalkName = pUser->GetNickName();
			}
		}
	}//lint !e429

	if(userVector.size() > 1)
	{
		qsort_s(&(userVector.at(0)), userVector.size(), sizeof(CChatUser*), CChatModel::ItemComareFunc, this);	
	}

	//restore username,nickname from database
	if (strLastMsgUserName.IsEmpty() || strLastMsgUserTalkName.IsEmpty())
	{
		CRecentMessageSqlite rms;
		BOOL bResSqlite3 = rms.Open();
		if (bResSqlite3)
		{
			BOOL bRes = rms.GetUserNameAndNickName(myRoomData.nRoomID,myRoomData.nLastMsgTrid,myRoomData.strLastMsgUserId.c_str(),strLastMsgUserName,strLastMsgUserTalkName);
			if (!bRes)
			{
				NTALK_LOG->Debug(L"[CHAT] restore username,nickname from database fail %d:%d:%s", 
                    myRoomData.nRoomID,myRoomData.nLastMsgTrid,myRoomData.strLastMsgUserId.c_str());
			}
		}

		if (bResSqlite3)
		{
			rms.Close();
		}

	}

	CChatDlg* pChatDlg = NULL;

	CSingleLock lock(&m_csFindCreatedDlg);
	lock.Lock();

	CSingleLock removeLock(&m_csDialogRemove);
	removeLock.Lock();	// Find   dialog  ʵ removeLock ´.

	pChatDlg = FindCreatedSessionChatDlg(userVector, myRoomData.strServerIP.c_str(), myRoomData.nServerPort, myRoomData.nRoomID);

	pChatDlg = CreateAbnormalTerminatedChatDialog(pChatDlg, userVector, removeLock, myRoomData, /*myUserID,*/ strLastMsgUserName, strLastMsgUserTalkName);

	if (removeLock.IsLocked())
		removeLock.Unlock();

	if (lock.IsLocked())
		lock.Unlock();

    //Note: don't delete the CChatUser in userVector here, will remain in CChatModel::m_mapUserList
    return pChatDlg;
}

NE_MYROOM_DATA* ChatDialogManager::GetRoomData(int nRoomID)
{
	NE_MYROOM_DATA* pRoomData = NULL;
	map<int, NE_MYROOM_DATA>::iterator it = m_mapRecentChatRoomData.find(nRoomID);
	if(it != m_mapRecentChatRoomData.end())
	{
		pRoomData = &((*it).second);
	}
	return pRoomData;
}

void ChatDialogManager::AddRecentChatRoomData(const NE_MYROOM_DATA& myRoomData)
{
	m_mapRecentChatRoomData[myRoomData.nRoomID] = myRoomData;
}

void ChatDialogManager::RequestAllRecentUserStatus()
{
	CSingleLock lock(&m_csFindCreatedDlg, TRUE);
	GET_MESSENGER_DLG()->BeginUpdateAllChatUsersInRoomList();
	map<int, NE_MYROOM_DATA>::const_iterator it = m_mapRecentChatRoomData.begin();
	for(; it != m_mapRecentChatRoomData.end(); ++it)
	{
		const NE_MYROOM_DATA& roomData = (*it).second;
		GET_MESSENGER_DLG()->UpdateAllChatUsersInRoomList(roomData.listChatMemberData);
	}
	
	GET_MESSENGER_DLG()->EndUpdateAllChatUsersInRoomList();
}

bool ChatDialogManager::CheckChatUserNumber( std::vector<CChatUser *> &vecUser )
{
	bool bReturn = true;

	if (vecUser.empty() == true)
	{
		ASSERT(FALSE);
		bReturn = false;    //lint !e527
	}

	if (vecUser.size() > MAX_CHAT_USER)
	{
		//AfxMessageBox(IDS_STRING157);
		NMessageBox(GET_MESSENGER_DLG(), LoadStringFromResource(IDS_STRING157));
		bReturn = false;
	} 

	return bReturn;
}

CChatDlg* ChatDialogManager::Create( CreateChatDialog *pCreateChatDialog, bool bSetFocus)
{
    TRACE(TEXT("%s %d, IsSingleChat=%d\n"), TEXT(__FUNCTION__), __LINE__, pCreateChatDialog->IsSingleChat());

    bool bNotCreated = true;

	GT_MY_USER_DATA myUserData;
	g_pMessengerActor->GetMyUserInfo(myUserData);
	LPCWSTR myUserID = myUserData.strUserID;

	CSingleLock lock(&m_csFindCreatedDlg);
	lock.Lock();
    CChatDlg* pChatDlg = NULL;
	if (pCreateChatDialog->IsSingleChat())
	{
		std::wstring userId = pCreateChatDialog->GetUserID();
		pChatDlg = NULL;
		pChatDlg = FindSingleChatDlg(userId.c_str());

		if (pChatDlg)
		{
			// ̹ 1:1 ȭ ִ  ã â Ȱȭ ָ ȴ.
			::ShowWindow(pChatDlg->GetHWND(), SW_SHOW);

			// ȭâ Ȱȭ ָ鼭 ȭ Ʈ ߰ش.
			if (GET_MESSENGER_DLG()->GetMainFrame())
				GET_MESSENGER_DLG()->GetMainFrame()->AddChatRoomList(pChatDlg);
			


			if (pChatDlg->IsIconic())
				::ShowWindow(pChatDlg->GetHWND(), SW_RESTORE);

			pChatDlg->EnableWindow(TRUE);
			pChatDlg->SetWindowTop();
			pChatDlg->SetForegroundWindowForce(pChatDlg->GetSafeHwnd());

			bNotCreated = false;
		}
		else if (userId.compare(myUserID) == 0)
		{
			NMessageBox(CWnd::FromHandle(GET_MESSENGER_DLG()->GetSafeHwnd()), LoadStringFromResource(IDS_STRING258));	// ڱ ڽŰ ȭ  ϴ.
			bNotCreated = false;
		}
	}

	if (bNotCreated && IsAvailableChatRoom())
	{
		std::vector<CChatUser *> vecUser;
		pCreateChatDialog->MakeChatUsers(myUserID, vecUser);

		if (CheckChatUserNumber(vecUser))
		{
			pChatDlg = CreateChatDlg(vecUser, MAKETYPE_CREATE);
			if (pChatDlg != NULL)
			{
				AddUserListChatDlg(pChatDlg, vecUser);
				pChatDlg->SetWindowTop();
				pChatDlg->SetForegroundWindowForce(pChatDlg->GetSafeHwnd());
	
			}
		}
	}
	
	lock.Unlock();
	return pChatDlg;
}

void ChatDialogManager::SetReadOnly( BOOL bStatus, CObject* pObject )
{
	CChatDlg* pDialog = (CChatDlg*) pObject;

	pDialog->SetReadOnly(bStatus, TRUE);
}

void ChatDialogManager::SetSessionRecoveryOfAllChatDialog( BOOL bTry )
{
	POSITION pos = m_listDialog.GetHeadPosition();
	while(pos)
	{
		CChatDlg *pDlg = (CChatDlg *)m_listDialog.GetNext(pos);
		pDlg->SetSessionRetryAvailable(bTry);
	}
}

void ChatDialogManager::InitChatRoomList()
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;
	vector <CChatDlg*> vecChatDlg;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if ((NULL==pDlg) || (!IsWindowVisible(pDlg->GetSafeHwnd())))
		{
			continue;
		}

		GET_MESSENGER_DLG()->GetMainFrame()->AddChatRoomList(pDlg);
	}

	map<int, NE_MYROOM_DATA>::const_iterator it = m_mapRecentChatRoomData.begin();
	for(; it != m_mapRecentChatRoomData.end(); ++it)
	{
		const NE_MYROOM_DATA& roomData = (*it).second;
		GET_MESSENGER_DLG()->GetMainFrame()->AddChatRoomList(roomData);
	}
}

//int __cdecl ChatDialogManager::ItemComareFunc(void *pvlocale,const void *elem1, const void *elem2)
//{
//	if (!pvlocale || !elem1 || !elem2)
//		return 0;
//
//	CChatDlg **pChatDlg1 = (CChatDlg**)elem1;
//	CChatDlg **pChatDlg2 = (CChatDlg**)elem2;
//
//	CTime time1 = (*pChatDlg1)->GetCreationTime();
//	CTime time2 = (*pChatDlg2)->GetCreationTime();
//    int nRet = 0;
//	if(time1 > time2)
//	{
//		nRet = -1;
//	}
//	else
//	{
//		nRet = 1;
//	}
//	return nRet;
//}

BOOL ChatDialogManager::ActivateChatDialog( int nRoomNo, BOOL bResize )
{
	if (m_listDialog.GetCount() <= 0)
		return TRUE;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		if (pDlg->GetRoomID() == nRoomNo)
		{
			if (pDlg->IsIconic()
				|| pDlg->IsZoomed())
			{
				::ShowWindow(pDlg->GetHWND(), SW_RESTORE);		
			}

			if (bResize)
			{
				pDlg->GetCWnd()->SetWindowPos(NULL, 0, 0, DEF_MIN_SIZE_WIDTH, DEF_MIN_SIZE_HEIGHT, SWP_NOMOVE);
			}

			CPoint adjustPt = GetInsertScreenPoint(pDlg);
			
			if (adjustPt.x < 0 && adjustPt.y < 0)
			{
				return FALSE;
			}
			else
			{
				pDlg->GetCWnd()->SetWindowPos(NULL, adjustPt.x, adjustPt.y, 0, 0, SWP_NOSIZE);
				pDlg->AdjustChatWindowToSearchResult();
			}

			break;
		}
	}

	return TRUE;
}

void ChatDialogManager::SetForegroundChatDialog(int nRoomNo)
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		if (pDlg->GetRoomID() == nRoomNo)
		{
			::ShowWindow(pDlg->GetHWND(), SW_SHOW);
			pDlg->SetWindowTop();
			pDlg->SetForegroundWindowForce(pDlg->GetSafeHwnd());

			break;
		}
	}
}

void ChatDialogManager::InitializeScreenRects( BOOL bInitialize )
{
	GET_MESSENGER_DLG()->GetWindowRect(&m_chatAlignRects[0]);

	if (bInitialize)
		m_offset = 0;

	CAMSWnd::GetMultiMonitorScreenRect(&m_screenRect);

	if (m_chatAlignRects[MAIN_AREA].top < m_screenRect.top)
		m_chatAlignRects[MAIN_AREA].top =  m_screenRect.top;

	if (m_chatAlignRects[MAIN_AREA].bottom >  m_screenRect.bottom)
		m_chatAlignRects[MAIN_AREA].bottom =  m_screenRect.bottom;

	// â ȭ  ʿ  â Y  , Ʒ  ȭ    
	if (IsHalf())	
	{
		m_chatAlignRects[MAIN_LEFT_AREA].top = m_chatAlignRects[MAIN_AREA].top + m_offset;
		m_chatAlignRects[MAIN_LEFT_AREA].left = m_chatAlignRects[MAIN_AREA].right;
		m_chatAlignRects[MAIN_LEFT_AREA].right = m_screenRect.right;
		m_chatAlignRects[MAIN_LEFT_AREA].bottom = m_screenRect.bottom;

		m_chatAlignRects[MAIN_RIGHT_AREA].top = m_chatAlignRects[MAIN_AREA].top + m_offset;
		m_chatAlignRects[MAIN_RIGHT_AREA].left = m_screenRect.left;
		m_chatAlignRects[MAIN_RIGHT_AREA].right = m_chatAlignRects[MAIN_AREA].left;
		m_chatAlignRects[MAIN_RIGHT_AREA].bottom = m_screenRect.bottom;
	}
	else
	{
		m_chatAlignRects[MAIN_LEFT_AREA].top = m_screenRect.top + m_offset;
		m_chatAlignRects[MAIN_LEFT_AREA].left = m_chatAlignRects[MAIN_AREA].right;
		m_chatAlignRects[MAIN_LEFT_AREA].right = m_screenRect.right;
		m_chatAlignRects[MAIN_LEFT_AREA].bottom = m_chatAlignRects[MAIN_AREA].bottom;

		m_chatAlignRects[MAIN_RIGHT_AREA].top = m_screenRect.top + m_offset;
		m_chatAlignRects[MAIN_RIGHT_AREA].left = m_screenRect.left;
		m_chatAlignRects[MAIN_RIGHT_AREA].right = m_chatAlignRects[MAIN_AREA].left;
		m_chatAlignRects[MAIN_RIGHT_AREA].bottom = m_chatAlignRects[MAIN_AREA].bottom;
	}
}

CPoint ChatDialogManager::GetInsertScreenPoint(CChatDlg *pDlg)
{
	CPoint resultPoint = m_chatAlignRects[MAIN_LEFT_AREA].TopLeft();

	if (NULL == pDlg)
		return resultPoint;

	BOOL bGotPosition = FALSE;
    CRect rectTarget(0, 0, 0, 0);
	GetWindowRect(pDlg->GetSafeHwnd(), &rectTarget);
	CRect rectTitle(rectTarget);
	rectTitle.bottom = rectTitle.top + pDlg->GetTitleHeight();

	if (m_chatAlignRects[MAIN_LEFT_AREA].Height() < rectTitle.Height())
	{
		InitializeScreenRects(TRUE);
	}

	// ä â ʺ о    
	if (!IsAvailableWidth(rectTarget) || pDlg->IsZoomed())
	{
		resultPoint.x = -1;
		resultPoint.y = -1;
		
		return resultPoint;
	}

	int nMinimumWidth = rectTarget.Width()/CHAT_DIALOG_ALIGN_RATIO;
	if (m_chatAlignRects[MAIN_LEFT_AREA].Width() >= nMinimumWidth)	// ȭâ 1/4 ̻   ִٸ װ ű.
	{
		bGotPosition = TRUE;
		resultPoint = m_chatAlignRects[MAIN_LEFT_AREA].TopLeft();

		m_chatAlignRects[MAIN_LEFT_AREA].left += rectTarget.Width();
		m_chatAlignRects[MAIN_AREA].right += rectTarget.Width();
	}
	else if (m_chatAlignRects[MAIN_RIGHT_AREA].Width() >= nMinimumWidth)
	{
		bGotPosition = TRUE;
		resultPoint.x = m_chatAlignRects[MAIN_RIGHT_AREA].right - rectTarget.Width(); 
		resultPoint.y = m_chatAlignRects[MAIN_RIGHT_AREA].top; 

		m_chatAlignRects[MAIN_RIGHT_AREA].right = resultPoint.x;
		m_chatAlignRects[MAIN_AREA].left = resultPoint.x;
	}

	if (FALSE == bGotPosition)
	{
		m_offset += rectTitle.Height();
		InitializeScreenRects(FALSE);

		return GetInsertScreenPoint(pDlg);
	}

	return resultPoint;
}

bool ChatDialogManager::IsAvailableWidth(CRect rtDlg)
{
	CRect rtMainDlg(0, 0, 0, 0);
	GET_MESSENGER_DLG()->GetWindowRect(&rtMainDlg);

	int nDlgWidth = rtDlg.Width();

	if (nDlgWidth > rtMainDlg.left &&
		nDlgWidth > m_screenRect.Width() - rtMainDlg.right)
		return false;

	return true;
}

BOOL ChatDialogManager::DisplaySearchChatRoomList(CString strSearchText)
{
	if (NULL == GET_MESSENGER_DLG()->GetMainFrame()->GetSafeHwnd())
		return FALSE;

	if (m_listDialog.GetCount() <= 0)
		return FALSE;

	vector<CString> vecQueries;
	SplitString_NotEmpty(strSearchText.Trim(), _T(' '), vecQueries);
	int queryNo = vecQueries.size();		

	CChatDlg* pDlg = NULL;
	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		CString strAllUserName = pDlg->GetAllUserNameFromList();//GetAllUserName();
		CString strAllUserNameLow = strAllUserName.MakeLower();

		//ȭ  ˻
		int nFind = 0;
		CString strQuery;

		for (int queryIndex = 0 ; queryIndex < queryNo ; ++queryIndex)
		{
			strQuery = vecQueries[queryIndex];
			CString strQueryLow = strQuery.MakeLower();
			if (FALSE == strQuery.IsEmpty() 
				&& -1 != strAllUserNameLow.Find(strQueryLow))
			{
				++nFind;
			}
			else
			{
				break;
			}
		}

		if (nFind != queryNo)
		{
			//ȭ  ˻
			if (FALSE == pDlg->IsInChat(strSearchText))
			{
				if (GET_MESSENGER_DLG()->GetMainFrame())
					GET_MESSENGER_DLG()->GetMainFrame()->RemoveChatRoom(pDlg->GetRoomID());
			}
			else
			{
				GET_MESSENGER_DLG()->GetMainFrame()->UpdateChatRoomList(pDlg);
				pDlg->GetCWnd()->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
			}
		}
	}
	return TRUE;
}

void ChatDialogManager::CloseChatDialog(int nRoomID)
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		if (pDlg->GetRoomID() == nRoomID)
		{
			pDlg->CloseChatDialog(pDlg->GetNeedRestoreChatRoom());
		}
	}
}



void ChatDialogManager::CloseChatDialogs(const CUIntArray& nRoomIDArray)
{
	int nCount = nRoomIDArray.GetCount();
	if (nCount <= 0)
		return;

	vector<int> vecRoomIDNotClosed;
	for (int i = 0 ; i < nCount ; ++i)
	{
		int nRoomID = nRoomIDArray.GetAt(i);	
		CChatDlg *pDlg= GET_APP()->m_pChatDialogManager->GetChatDialog(nRoomID);
		if ((pDlg != NULL) && (SENDACTION_FILE == pDlg->GetOngoingAction()))
		{
			vecRoomIDNotClosed.push_back(nRoomID);
		}
	}
	if (vecRoomIDNotClosed.size() > 0)
	{
		vector<int>::iterator it = vecRoomIDNotClosed.begin();
		for (;it != vecRoomIDNotClosed.end();it ++)
		{
			int nRoomID = *it;
			CChatDlg *pDlg= GET_APP()->m_pChatDialogManager->GetChatDialog(nRoomID);
			if (pDlg != NULL)
			{
				GET_APP()->m_pChatDialogManager->SetForegroundChatDialog(nRoomID);
			}
		}
	}
	if ((int)vecRoomIDNotClosed.size() < (int)nRoomIDArray.GetSize())
	{
		for (int i = 0;i< nRoomIDArray.GetCount();i++)
		{
			int nRoomID = nRoomIDArray.GetAt(i);
			vector<int>::iterator itr =  find(vecRoomIDNotClosed.begin(), vecRoomIDNotClosed.end(), nRoomID);
			if (itr == vecRoomIDNotClosed.end())
			{	
				CloseChatDialog(nRoomID);
				GET_MESSENGER_DLG()->GetMainFrame()->RemoveChatRoom(nRoomID);
			}

		}
	}	
	if (vecRoomIDNotClosed.size() > 0)
	{
		NMessageBox(AfxGetMainWnd(), LoadStringFromResource(IDS_STRINGCHATDLGBUSY), _T(""), MB_OK);

	} 

}
/*

void ChatDialogManager::CloseAllChatDialog()
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		pDlg->CloseChatDialog();
	}
}*/

BOOL ChatDialogManager::IsInside( CRect rectRegion, CRect rectTarget )
{
	if (rectRegion.Height() >= rectTarget.Height()
		&& rectRegion.Width() >= rectTarget.Width())
		return TRUE;

	return FALSE;
}

void ChatDialogManager::InitializeScreenRect( int index )
{
	if (index < 0 || index > 2)
		return;

	m_chatAlignRects[index].top = 0;
	m_chatAlignRects[index].left = 0;
	m_chatAlignRects[index].right = 0;
	m_chatAlignRects[index].bottom = 0;
}

void ChatDialogManager::InitializeSearchRect()
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		pDlg->ClearChatSearchResult();
	}
}

void ChatDialogManager::InvalidateChatDlg()
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg)
			continue;

		pDlg->OnChangeDisplayStyle();
		pDlg->ReSizeCtrl();
		pDlg->GetCWnd()->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ERASE | RDW_ALLCHILDREN);
	}
}

CChatDlg* ChatDialogManager::CreateAbnormalTerminatedChatDialog( CChatDlg* pChatDlg, std::vector<CChatUser *> &userVector, CSingleLock &removeLock, NE_MYROOM_DATA &myRoomData, /*LPCWSTR myUserID,*/ CString strLastMsgUserName, CString strLastMsgUserTalkName )
{
	// ̹  äù̶ CChatUser ޸ Ѵ.
	if( pChatDlg != NULL )
	{
		std::vector<CChatUser *>::iterator itr = userVector.begin();
		for( ; itr != userVector.end(); itr++ )
		{
			delete *itr;
		}
        userVector.clear();
	}

	if (pChatDlg == NULL)
	{
		removeLock.Unlock();

		NTALK_LOG->Debug(L"[CHAT] MAKETYPE_FORCEJOINED, old, %d", myRoomData.nRoomID);
		pChatDlg = CreateChatDlg(userVector, MAKETYPE_FORCEJOINED, 2);
		
		if (NULL != pChatDlg)
		{
			// ׷ȭε   밡 Ѹ   1:1 ȭ Ǳ  roomType  ٽ .
			if (myRoomData.nRoomType == CHATROOM_MULTITYPE)
				pChatDlg->SetGroupRoom(TRUE);

			pChatDlg->SetServerInfo(myRoomData.strServerIP.c_str(), myRoomData.nServerPort, myRoomData.nRoomID, _T("old"));
			pChatDlg->SetSessionReason(REASON_CREATE);
			pChatDlg->ResetCreateSessionCheckTimes();
			CTime lastmsgtime = myRoomData.nLastMsgTime;
			GetLocaleTime(&lastmsgtime); //roomlistcreation timeΪ׼԰Ϣıʱøchatdlg
            pChatDlg->SetCreationTime(lastmsgtime);
			//pChatDlg->SetHiddenByEscEtc(true);
			AddUserListChatDlg(pChatDlg, userVector);
			
			bool bShowChatDlg = true;

			// 1:1ȭ̰ ȭ ϳ ų(   )
			// ׷ε ȥ   濡 Ѵ.
			if(!myRoomData.strLastMsg.empty())
			{
				if ((!pChatDlg->IsGroupRoom() && strLastMsgUserName.IsEmpty()) ||
					(pChatDlg->IsGroupRoom() && pChatDlg->GetChatUserMapCount() == 0))
				{
					// ⼭ ChatSession   ʾұ     ϵ ÷׸ Ѵ.
					//Modify by sevin, do not close chat window when chat window have yourself only
					//pChatDlg->SetRemoveChatDlgWhenChatClose(true);
					//bShowChatDlg = false;
				}
			}	
			

			bool result = pChatDlg->NegotiateChatSession();

			if (result != TRUE)
			{
				pChatDlg->CloseChatDialog();
			}
			else if (bShowChatDlg)
			{
				//::ShowWindow(pChatDlg->GetHWND(), SW_HIDE);
				pChatDlg->EnableWindow(TRUE);
			
				if(!CreateAbnormalChatItem(pChatDlg, strLastMsgUserName, strLastMsgUserTalkName,myRoomData))
				{
					pChatDlg->CloseChatDialog();
					return NULL;
				}		
				MainFrame* pMainFrame = GET_MESSENGER_DLG()->GetMainFrame();
				if (NULL != pMainFrame && NULL != pMainFrame->GetSafeHwnd())
				{
					pMainFrame->AddChatRoomList(pChatDlg);
				}
				
			}
		}
	}
	return pChatDlg;
}


bool ChatDialogManager::CreateAbnormalChatItem(CChatDlg* pChatDlg, CString strLastMsgUserName,
						CString strLastMsgUserTalkName, NE_MYROOM_DATA &myRoomData)
{
	UIElement *pUIElement = NULL;
	CChatMainView *pChatMainView = pChatDlg->GetChatMainViewPtr();
	if (!pChatMainView)
	{
		return false;
	}

	TranslateFileInfoPtr pTransFileInfo(new TranslateFileInfo());
	chatroomlist_t listroom = myRoomData.listChatMemberData;

    if (myRoomData.msgType == DEF_FTP_MSG_TYPE)
    {
		if(listroom.size() > 0)
		{
			CRecentMessageSqlite &sqlite3 = pChatDlg->GetChatMainViewPtr()->GetRecentMessageSqlite();
			BOOL bResult = sqlite3.LoadTransFileInfo(myRoomData.nRoomID,myRoomData.strLastMsgUserId.c_str(),myRoomData.nLastMsgTrid,pTransFileInfo);

            if (bResult)
            {
                if (pTransFileInfo->m_Status != sUploadSuccess && pTransFileInfo->m_Status != sRecvSuccess
                    && pTransFileInfo->m_Status != sUploadCancle)
                {
                    if(pTransFileInfo->m_bUpload)
                    {
                        pUIElement = pChatDlg->MakeUploadFileJob(pTransFileInfo, false); 
                    }
                    else
                    {
                        pUIElement = pChatDlg->MakeDownloadFileJob(pTransFileInfo, false);
                    }

                    CTime time(myRoomData.nLastMsgTime);
                    GetLocaleTime(&time);
                    pChatDlg->SetCreationTime(time);
                    if(GET_MESSENGER_DLG()->GetMainFrame() != NULL)
                    {
                        GET_MESSENGER_DLG()->GetMainFrame()->UpdateChatRoomList(pChatDlg, TRUE);
                    }
                } 
                else
                {
                    CTime time(myRoomData.nLastMsgTime);
                    GetLocaleTime(&time);
                    pUIElement = pChatDlg->AddChatItem(myRoomData.strLastMsgUserId.c_str(), 
                        strLastMsgUserName, 
                        strLastMsgUserTalkName, time, 
                        myRoomData.strLastMsg.c_str(), 
                        pChatDlg->GetDefaultChatFont(), 
                        myRoomData.nLastMsgTrid, myRoomData.nLastMsgSeq,
                        myRoomData.msgType,pTransFileInfo, false);
                }
			}
			else
			{
				NTALK_LOG->Debug(L"[CHAT] CreateAbnormalChatItem sqlite3 fail: room_id:%d roomType:%d	msgType:%d	msgid:%d	userid:%s ",
					myRoomData.nRoomID,myRoomData.nRoomType,myRoomData.msgType,myRoomData.nLastMsgTrid,myRoomData.strLastMsgUserId.c_str());
			}
	
		}
    } 
    else
    {
		if(myRoomData.strLastMsg.empty())
		{
			return false;
		}
		CTime time(myRoomData.nLastMsgTime);
		GetLocaleTime(&time);
		pUIElement = pChatDlg->AddChatItem(myRoomData.strLastMsgUserId.c_str(), 
			strLastMsgUserName, 
			strLastMsgUserTalkName, time, 
			myRoomData.strLastMsg.c_str(), 
			pChatDlg->GetDefaultChatFont(), 
			myRoomData.nLastMsgTrid, myRoomData.nLastMsgSeq,
			myRoomData.msgType, CFSharePtr<TranslateFileInfo>(NULL), false);
    }

	if (pUIElement)
	{
		pUIElement->EnableRetryButton(FALSE);
		pChatMainView->EnableLastMessage(TRUE);
	}
	const CString &strMyId = pChatDlg->GetChatModel()->GetMyID();
	if (strMyId.CompareNoCase(myRoomData.strLastMsgUserId.c_str()) == 0)
	{
		int status = pChatMainView->GetRecentMessageSqlite().GetMsgStatus(myRoomData.nRoomID,strMyId,myRoomData.nLastMsgTrid);
		pChatDlg->GetChatModel()->UpdateMsgStatus(strMyId,myRoomData.nLastMsgTrid,status);

		CChatMainView *pMainView = pChatDlg->GetChatMainViewPtr();
		pMainView->CalcUIElementRect();
		pMainView->UpdateView();
	}
	
	pChatMainView->ShowRecentMessage();
	
//	pChatDlg->RemoveUnnecessaryMessage();

	return true;
}
BOOL ChatDialogManager::IsHalf()
{
	return m_chatAlignRects[MAIN_AREA].top <= m_screenRect.Height()/CHAT_DIALOG_HALF_HEIGHT;
}

void ChatDialogManager::UpdateBuddyStatus(LPCWSTR strBuddyID, int nTypeBits)
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg || NULL == pDlg->GetSafeHwnd())
			continue;

		pDlg->UpdateBuddyStatus(strBuddyID, nTypeBits);
	}
}

void ChatDialogManager::UpdateNotBuddyStatus(LPCWSTR strBuddyID, int nUserStatus, LPCWSTR strNick, int nIPTStatus, int nFlag)
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg || NULL == pDlg->GetSafeHwnd())
			continue;

		pDlg->UpdateNotBuddyStatus(strBuddyID, nUserStatus, strNick, nIPTStatus, nFlag);
	}
}

void ChatDialogManager::UpdateMyStatus(int nFlag, int nUserStatus, LPCWSTR strNick, int nIPTStatus)
{
	if (m_listDialog.GetCount() <= 0)
		return;

	GT_MY_USER_DATA myUserData;
	g_pMessengerActor->GetMyUserInfo(myUserData);
	LPCWSTR myUserID = myUserData.strUserID;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg || NULL == pDlg->GetSafeHwnd())
			continue;

		pDlg->UpdateNotBuddyStatus(myUserID, nUserStatus, strNick, nIPTStatus, nFlag);
	}
}



void CreateBuddyChatDialog::MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector )
{
	if(m_buddyVector.empty() == true)
		return;

	std::vector<const IGTBuddyUser*>::const_iterator iter = m_buddyVector.begin();
	for(; iter != m_buddyVector.end(); ++iter)
	{
		const IGTBuddyUser* pUser = *iter;
		if (std::wstring(myUserID).compare(pUser->GetUserID()) != 0)
		{
			CChatUser* pChatUser = new CChatUser();
			if (NULL != pChatUser)
			{
				TRACE("%s%d %x\n", __FUNCTION__, __LINE__, pChatUser);
				pChatUser->SetGTBuddyUser(pUser);
				pChatUser->SetInviteStatus(FALSE);
				userVector.push_back(pChatUser);
			}
		}
	}
}

void CreateMemberChatDialog::MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector )
{
	if(m_memberVector.empty() == true)
		return;

	std::vector<NE_MEMBER_DATA>::const_iterator iter = m_memberVector.begin();
	for(; iter != m_memberVector.end(); ++iter)
	{
		const NE_MEMBER_DATA data = *iter;
		if (myUserID != data.strUserId)
		{
			CChatUser* pChatUser = new CChatUser();
			if (NULL != pChatUser)
			{
				pChatUser->SetMemberData(data);
				pChatUser->SetInviteStatus(FALSE);
				userVector.push_back(pChatUser);
			}
		}//lint !e429
	}
}

void CreateSingleMemberChatDialog::MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector )
{
	ASSERT(std::wstring(myUserID).compare(m_strUserID) != 0);
	CChatUser* pChatUser = new CChatUser();
	if (NULL != pChatUser)
	{
		pChatUser->SetID(m_strUserID);
		pChatUser->SetName(m_strUserName);
		pChatUser->SetInviteStatus(FALSE);
		userVector.push_back(pChatUser);
	}
}//lint !e429

void CreateSingleBuddyChatDialog::MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector )
{
	ASSERT(std::wstring(myUserID).compare(m_pUser->GetUserID()) != 0);
	CChatUser* pChatUser = new CChatUser();
	if (NULL != pChatUser)
	{
		pChatUser->SetGTBuddyUser(m_pUser);
		pChatUser->SetInviteStatus(FALSE);
		userVector.push_back(pChatUser);
	}
}//lint !e429

void CreateUserIDArrayChatDialog::MakeChatUsers(LPCWSTR myUserID, std::vector<CChatUser*>& userVector )
{
	if(m_userIDArray.IsEmpty() == TRUE)
		return;

	for(int i = 0; i < m_userIDArray.GetCount(); i++)
	{
		if (std::wstring(myUserID).compare(m_userIDArray[i]) != 0)
		{
			CChatUser* pUser = new CChatUser();
			if (NULL != pUser)
			{
				pUser->SetID(m_userIDArray[i]);
				pUser->SetName(_T(""));
				pUser->SetNickName(_T(""));
				pUser->SetInviteStatus(FALSE);

				userVector.push_back(pUser);
			}
		}//lint !e429
	}
}

void ChatDialogManager::RemoveBuddy(LPCWSTR  strBuddyID )
{
	if (m_listDialog.GetCount() <= 0)
		return;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (NULL == pDlg || NULL == pDlg->GetSafeHwnd())
			continue;

		pDlg->RemoveItem(strBuddyID);
	}
}

CChatDlg* ChatDialogManager::FindChatDlgByHWND(HWND hWnd)
{
	CChatDlg* pDlg = NULL;
	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		pDlg = (CChatDlg*)m_listDialog.GetNext(pos);

		if (hWnd == pDlg->GetSafeHwnd())
		{
			return pDlg;
		}
	}

	return NULL;
}

CChatDlg* ChatDialogManager::GetChatDialog(int nRoomNo)
{
	if (m_listDialog.GetCount() <= 0)
		return NULL;

	CChatDlg* pDlg = NULL;

	for (POSITION pos = m_listDialog.GetHeadPosition(); pos != NULL;)
	{
		CChatDlg *p = (CChatDlg*)m_listDialog.GetNext(pos);

		if (p->GetRoomID() == nRoomNo)
		{
			pDlg = p;

			break;
		}
	}

	return pDlg;
}